use rocket::serde::json::Json;
use rocket::{get, post, State};
use serde::Deserialize;
use serde_json::json;

use crate::variant::Variant;
use crate::{
    db::ItemDB,
    routes::{api_error, FallibleApiResponse},
};

use super::{item_does_not_exist_error, variant_does_not_exist_error};

#[derive(Debug, Deserialize)]
pub struct DemandForm {
    uuid: String,
    destination: String,
    price: String,
}

#[post("/demand", data = "<f>")]
pub async fn demand_route(f: Json<DemandForm>) -> FallibleApiResponse {
    let uuid = Variant::demand(
        &f.uuid,
        f.price
            .clone()
            .try_into()
            .map_err(|()| api_error("Price malformed"))?,
        &f.destination,
    )
    .await
    .ok_or_else(|| api_error("Demand failed"))?;

    Ok(json!({"uuid": uuid}))
}

#[get("/item/<item_id>/<variant_id>/demand")]
pub async fn demand_log_route(
    item_id: &str,
    variant_id: &str,
    itemdb: &State<ItemDB>,
) -> FallibleApiResponse {
    let variant = itemdb
        .get_item(item_id)
        .ok_or_else(item_does_not_exist_error)?
        .variant(variant_id)
        .ok_or_else(variant_does_not_exist_error)?;

    let transactions = variant.demand_log().await;

    Ok(json!(transactions))
}
