mod demand;
mod error;
mod supply;

pub use demand::*;
pub use error::*;
use mongod::Model;
pub use supply::*;

use rocket::get;
use rocket::State;
use serde_json::json;

use crate::db::ItemDB;
use crate::transaction::Transaction;

use super::api_error;
use crate::routes::FallibleApiResponse;

/// Returns a JSON response with all items in the database.
#[get("/items")]
pub fn get_items_route(itemdb: &State<ItemDB>) -> serde_json::Value {
    let items = itemdb.items();
    json!({"items": items})
}

/// Return an API Response for an `Item`
#[get("/item/<item_id>")]
pub fn item_route(item_id: &str, itemdb: &State<ItemDB>) -> FallibleApiResponse {
    let item = itemdb
        .get_item(item_id)
        .ok_or_else(item_does_not_exist_error)?;
    Ok(item.api_json())
}

/// Returns all variants of an Item
#[get("/item/<item_id>/variants")]
pub fn item_variants_page(item_id: &str, itemdb: &State<ItemDB>) -> FallibleApiResponse {
    let item = itemdb
        .get_item(item_id)
        .ok_or_else(item_does_not_exist_error)?;
    let variants = item.get_variants();

    Ok(json!({
        "item": item_id,
        "variants": variants
    }))
}

/// Returns an API Response for a `Transaction`
#[get("/transaction/<transaction>")]
pub async fn transaction_route(transaction: &str) -> FallibleApiResponse {
    let t = Transaction::get(transaction)
        .await
        .ok_or_else(|| api_error("No transaction with this UUID"))?;
    Ok(t.api_json())
}
